use Test::More tests => 1;

BEGIN {
use_ok( 'POE::Component::Algorithm::Evolutionary' );
}

diag( "Testing POE::Component::Algorithm::Evolutionary $Poe::Component::Algorithm::Evolutionary::VERSION" );
